--- src/lib/file-dotlock.c.old	2004-05-27 18:56:27.000000000 +0300
+++ src/lib/file-dotlock.c	2004-05-27 18:59:08.000000000 +0300
@@ -168,13 +168,27 @@
 
 	dotlock_r->dev = st.st_dev;
 	dotlock_r->ino = st.st_ino;
-	dotlock_r->mtime = st.st_mtime;
 
 	if (close(fd) < 0) {
 		i_error("close(%s) failed: %m", lock_path);
 		(void)unlink(lock_path);
 		return -1;
 	}
+
+	/* some NFS implementations may have used cached mtime in previous
+	   fstat() call. Check again to avoid "dotlock was modified" errors. */
+	if (stat(lock_path, &st) < 0) {
+		i_error("stat(%s) failed: %m", lock_path);
+		return -1;
+	}
+	/* extra sanity check won't hurt.. */
+	if (st.st_dev != dotlock_r->dev ||
+	    st.st_ino != dotlock_r->ino) {
+		i_error("dotlock %s was immediately recreated under us",
+			lock_path);
+		return -1;
+	}
+	dotlock_r->mtime = st.st_mtime;
 	return 1;
 }
 
